// ———————————————————————————————————————————————————————————————————————————————————————————————
// Script réalisé par Emmanuel Côtez (emmanuel.cotez@mnhn.fr)
//
// GBIF : va chercher des informations via l'API du GBIF à partir du nom d'institution sélectionné
//
// v.0.2
//
// ———————————————————————————————————————————————————————————————————————————————————————————————

	// Un document doit être ouvert
	try { var myDocument = app.activeDocument; }
	catch (e) { alert("No opened article; please open an article (InDesign file) before running the script again") ; exit() ; }



// Fonctions

#include 'functionsEC.js';
#include 'extendables/extendables.jsx';
#include 'restix.jsx';

var count=0 ;

function style_exist(style_name) {
	// test l'existence d'un style et le créé s'il n'existe pas
	if(app.activeDocument.characterStyles.item (style_name) == null) { 
		app.activeDocument.characterStyles.add({name:style_name});
	}	
	return app.activeDocument.characterStyles.item(style_name);
}

function trim (str) {
	// function written by Peter Kahrel
	// available there: https://community.adobe.com/t5/indesign-discussions/trim-function-not-working-in-extendscript-is-there-a-way-to-replicate-the-functioning/td-p/7671885
    return str.replace(/^\s+/,'').replace(/\s+$/,'');
	}


myColorAdd(app.activeDocument, "Lien hypertexte", ColorModel.PROCESS, [0,0,255]); // Lien hypertexte (couleur RVB)
myColorAdd(app.activeDocument, "SCRIPT-COLOR-GrSciColl-red-error", ColorModel.PROCESS, [255,0,0]); // rouge (en erreur)
myStyleHyperlink=style_exist("Lien hypertexte");
myStyleHyperlink.fillColor="Lien hypertexte";

// L'acronyme doit être sélectionné
mySel = app.selection;

if ( mySel.length!=1 
|| (!mySel[0].properties.baseline)
|| (mySel[0].contents=="")
) {
alert("Sélectionnez un acronyme à lier");
test_selection=false;
exit();
} else {
// Récupération de la sélection
acronyms = mySel[0].contents;
}

// Recherche des liens GrSciColl en faisant appel à l'API GBIF

	var acroStr = acronyms ;
	
	// modèle de requête sur le GBIF : https://api.gbif.org/v1/grscicoll/institution/suggest?q=ANSP
	
	var yourUrl = "http://api.gbif.org/v1/grscicoll/institution/suggest?q="+acroStr ;
	request = {
	  url:yourUrl,
	  }
	var response = restix.fetch(request);

	var json_obj = JSON.parse(response.body);
	
	// prise en compte de réponses multiples	
	var noms_complets = new Array() ;
	for (j=0;j<json_obj.length;j++) {
		noms_complets[j] = json_obj[j]["name"];
		} // fin prise en compte de réponses multiples

	var acronym_ok_id = "" ;

	if (json_obj.length>0) {
		// Si au moins 1 résultat
		if (json_obj.length==1) {
			// Si un seul résultat
			if (json_obj[0]["code"]==acroStr) {
				// Vérification si l'acronyme correspond bien au résultat renvoyé
				acronym_ok_id = json_obj[0]["key"] ;
			}
		} else if (json_obj.length>1) {
			// Si plus d'un seul résultat, demander lequel choisir	
			// Choix du résultat dans une liste
			var msg="";
			 var myCaseList = new Array();
			 var myCase ;
			 for (m=0 ; m<noms_complets.length ; m++) {
				myCaseList[m] = noms_complets[m] ;
				msg = msg + noms_complets[m] + " / " ;
			 }

			 var myDialog = app.dialogs.add({name:"Il y a plus d'un résultat pour l'acronyme ["+acroStr+"] :"})
			 with(myDialog){
			  with(dialogColumns.add()){
			   with (dialogRows.add()) {
				with (dialogColumns.add()) {
				 staticTexts.add({staticLabel:"Sélectionnez l'acronyme à lier :"});
				}
				with (dialogColumns.add()) {
				 myCase = dropdowns.add({stringList:myCaseList,selectedIndex:0,minWidth:200});
				}
			   }
			  }
			 }
			 var myResult = myDialog.show();

			 if (myResult != true){
			  // L'utilisateur a cliqué sur "Annuler" :
			  // Mettre en rouge l'acronyme sélectionné :
			  // app.selection[0].strokeColor="SCRIPT-COLOR-GrSciColl-red-error";	
			  acronym_ok_id="";
			  myDialog.destroy();
			 } else {
			  theCase = myCase.selectedIndex;
			  acronym_ok_id = json_obj[theCase]["key"];
			  myDialog.destroy();
			  // Le résultat a été sélectionné
			 }
		} 
	} else {
		// Si aucun résultat n'a été trouvé
		// Mettre en rouge l'acronyme sélectionné :
		app.selection[0].strokeColor="SCRIPT-COLOR-GrSciColl-red-error";	
		acronym_ok_id="";
		}

	// Soit il y a 0 ou un seul résultat, soit il y en a plusieurs
	// Si plusieurs résultats, soit les acronymes correspondent exactement à la requête, soit non (json_obj[j]["code"]) 
	// Si plusieurs acronymes correspondent exactement à la requête, proposer le choix à l'utilisateur
	
	// Une fois l'acronyme trouvé, récupérer son ID (json_obj[j]["key"]) pour pouvoir créer le lien vers la fiche en ligne
	
	// Pour récupérer des informations sur l'institution :
	// https://api.gbif.org/v1/grscicoll/institution/key
	// ex. :
	// https://api.gbif.org/v1/grscicoll/institution/ad7d1a18-2ef8-4b2c-a302-5888fe9a2e1a
	
	// Lien vers la page en ligne correspondant à l'ID de l'institution :
	// https://registry.gbif.org/institution/key
	// ex. :
	// https://registry.gbif.org/institution/ad7d1a18-2ef8-4b2c-a302-5888fe9a2e1a
	
	if (acronym_ok_id!="") { 
		// acronym_ok_id contient l'ID validé par l'utilisateur de l'identifiant de l'institution
		try { 
			var url = "https://registry.gbif.org/institution/"+acronym_ok_id ;
			var myHyperlinkSource = app.activeDocument.hyperlinkTextSources.add(app.selection[0]) ;
			var myHyperlinkURLDestination = app.activeDocument.hyperlinkURLDestinations.add(url) ;
			var myHyperlink = app.activeDocument.hyperlinks.add(myHyperlinkSource, myHyperlinkURLDestination, {name: "GrSciColl_"+acroStr+" ("+(count)+")"}) ; 
			app.selection[0].appliedCharacterStyle="Lien hypertexte";
			count++;
			} catch (e) { 
				alert ("Problème pendant la création du lien\n\n"+e) ; 
				}
	}